@@ -1133,8 +1133,6 @@ fm_tree_model_get_column_type (GtkTreeModel *model, int index)
 		return GDK_TYPE_PIXBUF;
 	case FM_TREE_MODEL_FONT_STYLE_COLUMN:
 		return PANGO_TYPE_STYLE;
-	case FM_TREE_MODEL_FONT_WEIGHT_COLUMN:
-		return PANGO_TYPE_WEIGHT;
 	default:
 		g_assert_not_reached ();
 	}
@@ -1288,14 +1286,6 @@ fm_tree_model_get_value (GtkTreeModel *model, GtkTreeIter *iter, int column, GVa
 			g_value_set_enum (value, PANGO_STYLE_NORMAL);
 		}
 		break;
-	case FM_TREE_MODEL_FONT_WEIGHT_COLUMN:
-		g_value_init (value, PANGO_TYPE_STYLE);
-		if (node != NULL && node->parent == NULL) {
-			g_value_set_enum (value, PANGO_WEIGHT_BOLD);
-		} else {
-			g_value_set_enum (value, PANGO_WEIGHT_NORMAL);
-		}
-		break;
 	default:
 		g_assert_not_reached ();
 	}
@@ -1705,6 +1695,7 @@ fm_tree_model_set_show_hidden_files (FMTreeModel *model,
 		return;
 	}
 	model->details->show_hidden_files = show_hidden_files;
+	model->details->show_backup_files = show_hidden_files;
 	stop_monitoring (model);
 	if (!show_hidden_files) {
 		destroy_by_function (model, nautilus_file_is_hidden_file);
@@ -1712,25 +1703,6 @@ fm_tree_model_set_show_hidden_files (FMTreeModel *model,
 	schedule_monitoring_update (model);
 }
 
-void
-fm_tree_model_set_show_backup_files (FMTreeModel *model,
-					   gboolean show_backup_files)
-{
-	g_return_if_fail (FM_IS_TREE_MODEL (model));
-	g_return_if_fail (show_backup_files == FALSE || show_backup_files == TRUE);
-
-	show_backup_files = show_backup_files != FALSE;
-	if (model->details->show_backup_files == show_backup_files) {
-		return;
-	}
-	model->details->show_backup_files = show_backup_files;
-	stop_monitoring (model);
-	if (!show_backup_files) {
-		destroy_by_function (model, nautilus_file_is_backup_file);
-	}
-	schedule_monitoring_update (model);
-}
-
 static gboolean
 file_is_not_directory (NautilusFile *file)
 {
