@@ -34,27 +34,22 @@
 #include "nautilus-application.h"
 #include "nautilus-self-check-functions.h"
 #include "nautilus-window.h"
-#include <bonobo-activation/bonobo-activation.h>
-#include <bonobo/bonobo-main.h>
 #include <dlfcn.h>
 #include <signal.h>
 #include <eel/eel-debug.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-self-checks.h>
+#include <libegg/eggsmclient.h>
+#include <libegg/eggdesktopfile.h>
 #include <gdk/gdkx.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkiconfactory.h>
-#include <gtk/gtksignal.h>
-#include <gdk/gdk.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gio/gdesktopappinfo.h>
-#include <libgnome/gnome-init.h>
-#include <libgnomeui/gnome-ui-init.h>
-#include <libgnomeui/gnome-client.h>
 #include <libnautilus-private/nautilus-debug-log.h>
 #include <libnautilus-private/nautilus-directory-metafile.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-lib-self-check-functions.h>
+#include <libnautilus-private/nautilus-icon-names.h>
 #include <libxml/parser.h>
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
@@ -115,15 +110,6 @@ event_loop_unregister (GtkObject *object)
 	}
 }
 
-static gboolean
-initial_event_loop_needed (gpointer data)
-{
-	if (!is_event_loop_needed ()) {
-		eel_gtk_main_quit_all ();
-	}
-	return FALSE;
-}
-
 void
 nautilus_main_event_loop_register (GtkObject *object)
 {
@@ -143,67 +129,15 @@ nautilus_main_event_loop_quit (gboolean explicit)
 {
 	if (explicit) {
 		/* Explicit --quit, make sure we don't restart */
-		gnome_client_set_restart_style (gnome_master_client (),
-						GNOME_RESTART_IF_RUNNING);
+		/* TODO: With the old session we needed to set restart
+		   style to GNOME_RESTART_IF_RUNNING here, but i don't think we need
+		   that now since gnome-session doesn't restart apps except on startup. */
 	}
 	while (event_loop_registrants != NULL) {
 		gtk_object_destroy (event_loop_registrants->data);
 	}
 }
 
-/* Copied from libnautilus/nautilus-program-choosing.c; In this case,
- * though, it's really needed because we have no real alternative when
- * no DESKTOP_STARTUP_ID (with its accompanying timestamp) is
- * provided...
- */
-static Time
-slowly_and_stupidly_obtain_timestamp (Display *xdisplay)
-{
-	Window xwindow;
-	XEvent event;
-	
-	{
-		XSetWindowAttributes attrs;
-		Atom atom_name;
-		Atom atom_type;
-		char* name;
-		
-		attrs.override_redirect = True;
-		attrs.event_mask = PropertyChangeMask | StructureNotifyMask;
-		
-		xwindow =
-			XCreateWindow (xdisplay,
-				       RootWindow (xdisplay, 0),
-				       -100, -100, 1, 1,
-				       0,
-				       CopyFromParent,
-				       CopyFromParent,
-				       (Visual *)CopyFromParent,
-				       CWOverrideRedirect | CWEventMask,
-				       &attrs);
-		
-		atom_name = XInternAtom (xdisplay, "WM_NAME", TRUE);
-		g_assert (atom_name != None);
-		atom_type = XInternAtom (xdisplay, "STRING", TRUE);
-		g_assert (atom_type != None);
-		
-		name = "Fake Window";
-		XChangeProperty (xdisplay, 
-				 xwindow, atom_name,
-				 atom_type,
-				 8, PropModeReplace, name, strlen (name));
-	}
-	
-	XWindowEvent (xdisplay,
-		      xwindow,
-		      PropertyChangeMask,
-		      &event);
-	
-	XDestroyWindow(xdisplay, xwindow);
-	
-	return event.xproperty.time;
-}
-
 static void
 dump_debug_log (void)
 {
@@ -368,26 +302,31 @@ int
 main (int argc, char *argv[])
 {
 	gboolean kill_shell;
-	gboolean restart_shell;
 	gboolean no_default_window;
 	gboolean browser_window;
 	gboolean no_desktop;
-	const char *startup_id;
-	char *startup_id_copy;
-	char *session_to_load;
+	gboolean version;
+	gboolean autostart_mode;
+	const char *autostart_id;
 	gchar *geometry;
 	const gchar **remaining;
 	gboolean perform_self_check;
-	GOptionContext *context;
 	NautilusApplication *application;
-	char **argv_copy;
-	GnomeProgram *program;
+	GOptionContext *context;
+	GFile *file;
+	char *uri;
+	char **uris;
+	GPtrArray *uris_array;
+	GError *error;
+	int i;
 	
 	const GOptionEntry options[] = {
 #ifndef NAUTILUS_OMIT_SELF_CHECK
 		{ "check", 'c', 0, G_OPTION_ARG_NONE, &perform_self_check, 
 		  N_("Perform a quick set of self-check tests."), NULL },
 #endif
+		{ "version", '\0', 0, G_OPTION_ARG_NONE, &version,
+		  N_("Show the version of the progam."), NULL },
 		{ "geometry", 'g', 0, G_OPTION_ARG_STRING, &geometry,
 		  N_("Create the initial window with the given geometry."), N_("GEOMETRY") },
 		{ "no-default-window", 'n', 0, G_OPTION_ARG_NONE, &no_default_window,
@@ -398,20 +337,13 @@ main (int argc, char *argv[])
 		  N_("open a browser window."), NULL },
 		{ "quit", 'q', 0, G_OPTION_ARG_NONE, &kill_shell, 
 		  N_("Quit Nautilus."), NULL },
-		{ "restart", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &restart_shell,
-		  N_("Restart Nautilus."), NULL },
 		{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &remaining, NULL,  N_("[URI...]") },
-		{ "load-session", 'l', 0, G_OPTION_ARG_STRING, &session_to_load,
-		  /* Translators: --no-default-window is a nautilus command line parameter, don't modify it. */
-		  N_("Load a saved session from the specified file. Implies \"--no-default-window\"."), N_("FILENAME") },
 
 		{ NULL }
 	};
 
 	g_thread_init (NULL);
 
-	setlocale (LC_ALL, "");
-
 	/* This will be done by gtk+ later, but for now, force it to GNOME */
 	g_desktop_app_info_set_desktop_env ("GNOME");
 
@@ -424,79 +356,69 @@ main (int argc, char *argv[])
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	startup_id = g_getenv ("DESKTOP_STARTUP_ID");
-	startup_id_copy = NULL;
-	if (startup_id != NULL && *startup_id != '\0') {
-		/* Clear the DESKTOP_STARTUP_ID, but make sure to copy it first */
-		startup_id_copy = g_strdup (startup_id);
-		g_unsetenv ("DESKTOP_STARTUP_ID");
-	}
+	autostart_mode = FALSE;
 
-	/* we'll do it ourselves due to complicated factory setup */
-	gtk_window_set_auto_startup_notification (FALSE);
+	autostart_id = g_getenv ("DESKTOP_AUTOSTART_ID");
+	if (autostart_id != NULL && *autostart_id != '\0') {
+		autostart_mode = TRUE;
+        }
 
 	/* Get parameters. */
 	remaining = NULL;
 	geometry = NULL;
-	session_to_load = NULL;
+	version = FALSE;
 	kill_shell = FALSE;
 	no_default_window = FALSE;
 	no_desktop = FALSE;
 	perform_self_check = FALSE;
-	restart_shell = FALSE;
 	browser_window = FALSE;
 
-	g_set_application_name (_("File Manager"));
+	g_set_prgname ("nautilus");
+	if (g_file_test (DATADIR "/applications/nautilus.desktop", G_FILE_TEST_EXISTS)) {
+		egg_set_desktop_file (DATADIR "/applications/nautilus.desktop");
+	}
+
 	context = g_option_context_new (_("\n\nBrowse the file system with the file manager"));
+	g_option_context_add_main_entries (context, options, NULL);
+	
+	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+	g_option_context_add_group (context, egg_sm_client_get_option_group ());
+	
+	error = NULL;
+	if (!g_option_context_parse (context, &argc, &argv, &error)) {
+		g_printerr ("Could not parse arguments: %s\n", error->message);
+		g_error_free (error);
+		return 1;
+	}
 
-	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+	g_option_context_free (context);
 
+	if (version) {
+		g_print ("GNOME nautilus " PACKAGE_VERSION "\n");
+		return 0;
+	}
+	
 #ifdef HAVE_EXEMPI
 	xmp_init();
 #endif
 
-	program = gnome_program_init ("nautilus", VERSION,
-				      LIBGNOMEUI_MODULE, argc, argv,
-				      GNOME_PROGRAM_STANDARD_PROPERTIES,
-				      GNOME_PARAM_GOPTION_CONTEXT, context,
-				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Nautilus"),
-				      NULL);
-
-	/* We do this after gnome_program_init(), since that function sets up
-	 * its own handler for SIGSEGV and others --- we want to chain to those
-	 * handlers.
-	 */
 	setup_debug_log ();
 
-	if (session_to_load != NULL) {
+	/* If in autostart mode (aka started by gnome-session), we need to ensure 
+         * nautilus starts with the correct options.
+         */
+	if (autostart_mode) {
 		no_default_window = TRUE;
+		no_desktop = FALSE;
 	}
 
-	/* Do this here so that gdk_display is initialized */
-	if (startup_id_copy == NULL) {
-		/* Create a fake one containing a timestamp that we can use */
-		Time timestamp;
-		timestamp = slowly_and_stupidly_obtain_timestamp (gdk_display);
-		startup_id_copy = g_strdup_printf ("_TIME%lu",
-						   timestamp);
-	}
-
-        /* Set default icon for all nautilus windows */
-	gtk_window_set_default_icon_name ("gnome-fs-directory");
-	
-	/* Need to set this to the canonical DISPLAY value, since
-	   thats where we're registering per-display components */
-	bonobo_activation_set_activation_env_value ("DISPLAY",
-						    gdk_display_get_name (gdk_display_get_default()));
-	
-
 	if (perform_self_check && remaining != NULL) {
 		/* translators: %s is an option (e.g. --check) */
 		fprintf (stderr, _("nautilus: %s cannot be used with URIs.\n"),
 			"--check");
 		return EXIT_FAILURE;
 	}
-	if (perform_self_check && (kill_shell || restart_shell)) {
+	if (perform_self_check && kill_shell) {
 		fprintf (stderr, _("nautilus: --check cannot be used with other options.\n"));
 		return EXIT_FAILURE;
 	}
@@ -505,11 +427,6 @@ main (int argc, char *argv[])
 			"--quit");
 		return EXIT_FAILURE;
 	}
-	if (restart_shell && remaining != NULL) {
-		fprintf (stderr, _("nautilus: %s cannot be used with URIs.\n"),
-			"--restart");
-		return EXIT_FAILURE;
-	}
 	if (geometry != NULL && remaining != NULL && remaining[0] != NULL && remaining[1] != NULL) {
 		fprintf (stderr, _("nautilus: --geometry cannot be used with more than one URI.\n"));
 		return EXIT_FAILURE;
@@ -529,11 +446,9 @@ main (int argc, char *argv[])
 		eel_preferences_set_is_invisible
 			(NAUTILUS_PREFERENCES_DESKTOP_IS_HOME_DIR, TRUE);
 	}
-	
-	bonobo_activate (); /* do now since we need it before main loop */
 
 	application = NULL;
- 
+
 	/* Do either the self-check or the real work. */
 	if (perform_self_check) {
 #ifndef NAUTILUS_OMIT_SELF_CHECK
@@ -548,53 +463,52 @@ main (int argc, char *argv[])
 		eel_exit_if_self_checks_failed ();
 #endif
 	} else {
+		/* Convert args to URIs */
+		uris = NULL;
+		if (remaining != NULL) {
+			uris_array = g_ptr_array_new ();
+			for (i = 0; remaining[i] != NULL; i++) {
+				file = g_file_new_for_commandline_arg (remaining[i]);
+				if (file != NULL) {
+					uri = g_file_get_uri (file);
+					g_object_unref (file);
+					if (uri) {
+						g_ptr_array_add (uris_array, uri);
+					}
+				}
+			}
+			g_ptr_array_add (uris_array, NULL);
+			uris = (char **)g_ptr_array_free (uris_array, FALSE);
+		}
+
+		
 		/* Run the nautilus application. */
 		application = nautilus_application_new ();
+
+		if (egg_sm_client_is_resumed (application->smclient)) {
+			no_default_window = TRUE;
+		}
+		
 		nautilus_application_startup
 			(application,
-			 kill_shell, restart_shell, no_default_window, no_desktop,
-			 !(kill_shell || restart_shell),
+			 kill_shell, no_default_window, no_desktop,
 			 browser_window,
-			 startup_id_copy,
 			 geometry,
-			 session_to_load,
-			 remaining);
-		g_free (startup_id_copy);
+			 uris);
+		g_strfreev (uris);
 
-		/* The application startup does things in an idle, so
-		   we need to check whether the main loop is needed in an idle
-		*/
-		g_idle_add (initial_event_loop_needed, NULL);
-		gtk_main ();
+		if (is_event_loop_needed ()) {
+			gtk_main ();
+		}
+	}
+
+	nautilus_icon_info_clear_caches ();
+	
+	if (application != NULL) {
+		g_object_unref (application);
 	}
 
  	eel_debug_shut_down ();
-
-	if (application != NULL) {
-		bonobo_object_unref (application);
-	}
-
-	/* If told to restart, exec() myself again. This is used when
-	 * the program is told to restart with CORBA, for example when
-	 * an update takes place.
-	 */
-
-	if (g_getenv ("_NAUTILUS_RESTART") != NULL) {
-		g_unsetenv ("_NAUTILUS_RESTART");
-		
-		/* Might eventually want to copy all the parameters
-		 * from argv into the new exec. For now, though, that
-		 * would just interfere with the re-creation of
-		 * windows based on the window info stored in gconf,
-		 * including whether the desktop was started.
-		 */
-		argv_copy = g_new0 (char *, 2);
-		argv_copy[0] = argv[0];
-		
-		execvp (argv[0], argv_copy);
-	}
-
-	g_object_unref (G_OBJECT (program));
-
+	
 	return EXIT_SUCCESS;
 }
