@@ -30,8 +30,7 @@
 #ifndef NAUTILUS_NAVIGATION_WINDOW_H
 #define NAUTILUS_NAVIGATION_WINDOW_H
 
-#include <bonobo/bonobo-window.h>
-#include <gtk/gtktoolitem.h>
+#include <gtk/gtk.h>
 #include <eel/eel-glib-extensions.h>
 #include <libnautilus-private/nautilus-bookmark.h>
 #include <libnautilus-private/nautilus-sidebar.h>
@@ -50,6 +49,7 @@ typedef struct _NautilusNavigationWindow        NautilusNavigationWindow;
 typedef struct _NautilusNavigationWindowClass   NautilusNavigationWindowClass;
 typedef struct _NautilusNavigationWindowDetails NautilusNavigationWindowDetails; 
 
+
 struct _NautilusNavigationWindow {
         NautilusWindow parent_object;
         
@@ -61,19 +61,13 @@ struct _NautilusNavigationWindow {
         GtkWidget *navigation_bar;
 	GtkWidget *path_bar;
         GtkWidget *search_bar;
-       
-        /* Back/Forward chain, and history list. 
-         * The data in these lists are NautilusBookmark pointers. 
-         */
-        GList *back_list, *forward_list;
-        
+	GtkWidget *notebook;
+
         /* Current views stuff */
         GList *sidebar_panels;
         
         /* Widgets to keep track of (for state changes, etc) */      
         GtkWidget *zoom_control;
-
-	GtkTooltips *tooltips;
 };
 
 
@@ -82,7 +76,6 @@ struct _NautilusNavigationWindowClass {
 };
 
 GType    nautilus_navigation_window_get_type             (void);
-void     nautilus_navigation_window_go_home              (NautilusNavigationWindow *window);
 void     nautilus_navigation_window_allow_back           (NautilusNavigationWindow *window,
                                                           gboolean                  allow);
 void     nautilus_navigation_window_allow_forward        (NautilusNavigationWindow *window,
@@ -118,7 +111,8 @@ void     nautilus_navigation_window_show_status_bar      (NautilusNavigationWind
 gboolean nautilus_navigation_window_status_bar_showing   (NautilusNavigationWindow *window);
 void     nautilus_navigation_window_back_or_forward      (NautilusNavigationWindow *window,
                                                           gboolean                  back,
-                                                          guint                     distance);
+                                                          guint                     distance,
+							  gboolean                  new_tab);
 void     nautilus_navigation_window_show_search          (NautilusNavigationWindow *window);
 
 #endif
